/** Class for FracLac calculates statistics on passed arrays.
 * Can remove areas of horizontal slope from the array,
 * and return alternate values (see constructors).
 * This program is free software distributed in the same way that ImageJ is.
 *
 * A. Karperien, Charles Sturt University, August 2003
 * Copyright (C) 2003 Audrey Karperien
 * For futher information or to send comments, bugs, and feature requests
 * contact{@link <a href="mailto:akarpe@postoffice.csu.edu.au"> the author</a>}
 * or see
 * {@link <a href="http://www.geocities.com/akarpe@sbcglobal.net/box.html">
 * FracLac page</a>}.
 * <p>file: FracLac_2003August__1.java</p>
 * <p>IDE: Sun ONE Studio 4, Community Edition</p>
 * @version FracLac2003: Version 2.4 August 31, 2003
 * @author Audrey Karperien (akarpe01@postoffice.csu.edu.au)
 * @since J2SKD1.4.1_02b
 */

public class FracStats
{
    //////////////////////////////////////////////////////////////////////
    //////////////////////////// Variables  //////////////////////////////
    //////////////////////////////////////////////////////////////////////
    public float[] oldcount;
    
    public int[] oldsize;
    
    //public  double[] CDF;
    
    public int correctedNumBoxes = 0;
    
    public int oldnum;
    
    public int[] CSize;
    
    public float[] CCount;
    
    /**Fractal dimension or negative slope of log-log regression line*/
    public double df;
    /**Y intercept of log-log regression line.*/
    public double yint;
    /**F parameter*/
    public double F;
    /**{@link #FindCompressed Compressed} fractal dimension.*/
    public double cdf;
    /**{@link #FindCompressed Compressed} y-intercept.*/
    public double cyint;
    /**{@link #FindCompressed Compressed} F parameter.*/
    public double cF;
    /**R squared for log-log regression line.*/
    public double rsq;
    /**R squared for  {@link #FindCompressed Compressed} regression line.*/
    public  double crsq;
    /**Slope of linear regression line.*/
    public double ldf;
    /**Y-intercept of linear regression line.*/
    public double lyint;
    /**R squared for linear regression line.*/
    public  double lrsq;
    /**F parameter for linear regression line.*/
    public double lF;
    /**Standard error of log-log regression line.*/
    public double stderr;
    /**Standard error of linear regression line.*/
    public double lstderr;
    /**Standard error compressed log-log regression line.*/
    public   double cstderr;
    /**Linear regression line compressed slope.*/
    public  double lcdf;
    /**R squared for linear  @link #FindCompressed Compressed} regression line.*/
    public double lcrsq;
    /**Y-intercept for linear {@link #FindCompressed Compressed} regression line.*/
    public double lcyint;
    /**F parameter for linear  {@link #FindCompressed Compressed} regression line.*/
    public double lcF;
    /**Standard error for  {@link #FindCompressed Compressed} linear regression line.*/
    public double lcstderr;
    //////////////////////////////////////////////////////////////////////
    ////////////////////////////Constructors//////////////////////////////
    //////////////////////////////////////////////////////////////////////
    /**Constructor accepts arrays and calls {@link #GetStats GetStats} method.
     *@param C integer array (corresponds to counts for fractal analysis
     * @param S integer array (corresponds to sizes for fractal analysis
     * @param originalNumSizes number of values to use in passed arrays
     */
    public FracStats(int[] C, int[] S, int originalNumSizes)
    {
        oldcount = new float [originalNumSizes];
        for (int w = 0; w<originalNumSizes; w++)
        {oldcount[w]=(float)C[w];}
        this.oldsize=S;
        this.oldnum=originalNumSizes;
        GetStats(oldcount, oldsize, originalNumSizes);
        
    }
    
    /**Constructor accepts arrays and calls {@link #GetStats GetStats} method.
     *@param C float array (corresponds to counts for fractal analysis
     * @param S integer array (corresponds to sizes for fractal analysis
     * @param originalNumSizes number of values to use in passed arrays
     */
    public FracStats(float[] C, int[] S, int originalNumSizes)
    {
        this.oldsize=S;
        this.oldcount=C;
        this.oldnum=originalNumSizes;
        GetStats(oldcount, oldsize, originalNumSizes);
        
        
    }
    //////////////////////////////////////////////////////////////////////
    ////////////////////////////  Methods   //////////////////////////////
    //////////////////////////////////////////////////////////////////////
    /**Gets all stats on the data. Must be called to read values properly;
     * is automatically called by constructors but must be
     * called again for any changes. Sets variables after calling
     *{@link #FindDfStats FindDfStats}, {@link #FindCompressed FindCompressed},
     *and {@link #LinearRegression LinearRegression} methods.
     * Variables set are {@link #df}, {@link #rsq}, {@link #yint},
     * {@link #F}, {@link #stderr}, {@link #cdf}, {@link #crsq}, {@link #cyint},
     * {@link #cF}, {@link #cstderr}, {@link #ldf}, {@link #lrsq}, {@link #lyint},
     * {@link #lF}, {@link #lstderr}, {@link #lcdf}, {@link #lcrsq}, {@link #lcyint},
     * {@link #lcF}, {@link #lcstderr}.
     *
     * @param oldcount float array of counts
     * @param oldsize integer array of sizes
     * @param originalNumSizes integer for number of values to use
     */
    public void GetStats(float[] oldcount, int[] oldsize, int originalNumSizes)
    {
        double [] reg = FindDfStats(oldcount, oldsize, originalNumSizes);
        df= reg[0];
        rsq=reg[1];
        yint=reg[2];
        F=reg[3];
        stderr=reg[4];
        double [] comp = FindCompressed(oldcount, oldsize, originalNumSizes);
        cdf= comp[0];
        crsq=comp[1];
        cyint=comp[2];
        cF=comp[3];
        cstderr=comp[4];
        double [] linear
        = LinearRegression(oldcount, oldsize, originalNumSizes);
        ldf=linear[0];
        lrsq=linear[1];
        lyint=linear[2];
        lF=linear[3];
        lstderr=linear[4];
        double [] lcompress = LinearRegression(CCount, CSize, correctedNumBoxes);
        lcdf=lcompress[0];
        lcrsq=lcompress[1];
        lcyint=lcompress[2];
        lcF=lcompress[3];
        lcstderr=lcompress[4];
    }
    
    /** Removes areas of horizontal slope and returns
     * {@link #FindDfStats array} for
     * negative slope of regression line (box count fractal
     * dimension), y intercept, and correlation coefficient
     * from log-log plot of passed arrays. Calls an overloaded
     * version of FindCompressed that accepts a float array
     * after converting the Count array of integers to floats.
     * @param testarray is the first integer array to assess (e.g., of counts
     * of boxes containing pixels in a box count)
     * @param Sizes is an integer array, the second array to
     * assess in calculating regression line
     * @param oldNumSizes int for the number of entries to assess
     * in the passed arrays
     * @return double array with 3 entries for negative
     * slope of regression line (box count fractal dimension), y intercept,
     * and correlation coefficient from regression line for
     * log-log plot of compressed arrays
     */
    public double[] FindCompressed(int[] intCount, int[] intSizes, int oldNmSzs)
    {/**converts counts to floats array then calls stats to make and use compressed array*/
        float []floatcount = new float [oldNmSzs];
        for (int w = 0; w<oldNmSzs; w++)
        {floatcount[w]=(float)intCount[w];}
        return FindCompressed(floatcount, intSizes, oldNmSzs);
    }
    
    /** Removes areas of horizontal slope and returns
     * {@link #FindDfStats array}.
     * @param testarray is the first float array to assess (e.g., of counts
     * of boxes containing pixels in a box count)
     * @param Sizes is an integer array, the second array to
     * assess in calculating regression line
     * @param oldNumSizes int for the number of entries to assess
     * in the passed arrays
     * @return double array from {@link #FindDfStats FindDfStats method} */
    public double[] FindCompressed(float[] testarray, int[] Sizes, int oldNumSizes)
    {/*shrinks array then calls function to find stats*/
        /*calculates corrected fractal dimension without horizontal slopes*/
        /*uses lowest box size in horizontal part*/
        correctedNumBoxes = 0;
        double CsumSC=0, CsumS=0, CsumC=0, CsumSsq = 0, CsumCsq;
        CSize=new int [oldNumSizes];
        CCount = new float [oldNumSizes];
        for (int q=0; q<oldNumSizes; q++)
            //for (int q=NumSizes-1;q>=0; q--)
        {
            boolean proceed = false;
            if (q>0)
            {
                if (testarray[q]<testarray[q-1])proceed=true;
            }
            if (proceed==true)
            {
                CSize[correctedNumBoxes]=Sizes[q];
                CCount[correctedNumBoxes]=testarray[q];
                correctedNumBoxes++;
            }
        }
        return FindDfStats(CCount, CSize, correctedNumBoxes);
        
    }
    
    /**Calculates stats using the log-log plot of Counts and the ratio
     * of Sizes/Biggest Size.  See {@link #FindDfStats FindDfStats}
     *for further calculations.
     * @param Counts float array
     * @param Sizes integer array
     * @param NumSizes integer for number of values to use
     * @return double array of statistics as the negative slope
     * of the regressionline or fractal dimension, the rsquared
     *for the regression line, the y-intercept, the inverse of the prefactor,
     *and the standard error.
     */
    public double[] ratioFindDfStats(float[] Counts, int[] Sizes, int NumSizes)
    {  /*takes an array for box sizes and an array for Counts and
     *the number of box sizes to check and calculates stats using those arrays*/
        double n=(double)NumSizes;
        double biggestSize = (double)Sizes[0];
        for (int q = 0; q<NumSizes; q++)
        {
            biggestSize = Math.max((double)Sizes[q], biggestSize);
        }
        double DsumSC=0, DsumS=0, DsumC=0, DsumSsq = 0, DsumCsq=0;
        for (int q=0; q<NumSizes; q++)
        {/*go through array and sum values for regression line*/
            /////SUM THE LOG OF THE COUNT*THE LOG OF THE SIZE\
            ////use the scale factor, calculated as the
            ////box size divided by the largest size
            //That is, the maximum size is treated as 1 now
            DsumSC
            = DsumSC + (log(((double)Sizes[q])/biggestSize)
            *log((double)Counts[q]));
            ///SUM THE LOG OF THE SIZE
            DsumS = DsumS + log(((double)Sizes[q])/biggestSize);
            //SUM THE LOG OF THE COUNTS
            DsumC = DsumC + log((double)Counts[q]);
            //SUM THE LOG OF COUNTS TIMES LOG OF COUNTS
            DsumCsq
            =DsumCsq+(log((double)Counts[q])*log((double)Counts[q]));
            //SUM THE LOG OF SIZE TIMES LOG SIZE
            DsumSsq
            = DsumSsq
            + (log(((double)Sizes[q])/biggestSize)
            *log(((double)Sizes[q])/biggestSize));
        }
        //CALCULATE THE SLOPE OF THE REGRESSION LINE
        double b=
        ((n*DsumSC)-(DsumS*DsumC))
        /((n*DsumSsq)-(DsumS*DsumS));
        double adf=b;//-1*b;
        //CALCULATE CORRELATION COEFFICIENT OF REGRESSION LINE
        double arSq= (n*(DsumSC)-(DsumS*DsumC))
        /Math.sqrt(((n*DsumSsq)-(DsumS*DsumS))
        *((n*DsumCsq)-((DsumC*DsumC))));
        
        arSq=arSq*arSq;
        
        //CALCULATE Y INTERCEPT OF REGRESSION LINE
        double ayint
        = (DsumC - (b*DsumS))/n;
        /*y intercept for linear regression by least squares*/
        //System.out.println(ayint);
        double F=Math.exp(ayint);//Prefactor A = e to the yint
        //for a power law: y=Ax^Df
        double StandardError=Math.sqrt((DsumCsq-ayint*DsumC-b*DsumSC)/(n-2));
        double [] DFinfo = new double[5];
        DFinfo[0]=adf;
        DFinfo[1]=arSq;
        DFinfo[2]=ayint;
        DFinfo[3]=1.0/F;//this is the inverse of the prefactor
        DFinfo[4]=StandardError;
        return DFinfo;
        
        
    }
    
    /** Returns array containing power law regression data, using
     * Counts (Y) array and 1/Sizes (1/X) array.  Regression equation is calculated
     * as usual linear regression using log of passed arrays.
     * Df=NumSizes(SumXY) -(SumX)(SumY)/(NumSizes(SumXsquared)-SumX*SumX)
     * and
     * a=(SumY-Df(SumX))/NumSizes
     * @param Counts array of floats
     * @param Sizes array of integers; 1/Sizes[i] is used
     * @param NumSizes an integer for the number of values to use
     * from the passed arrays
     * @return double array with array[0]=fractal dimension or slope
     * , array[1]=r squared correlation coefficient; array[2]=y intercept
     * for linear equation y =(Df)*x + b; array[3]=1/F, where F=euler's number
     * ^y intercept, array[4]=standard error */
    public double[] FindDfStats(float[] Counts, int[] Sizes, int NumSizes)
    {  /*takes an array for box sizes and an array for Counts and
     *the number of box sizes to check and calculates stats using those arrays*/
        double n=(double)NumSizes;
        double DsumSC=0, DsumS=0, DsumC=0, DsumSsq = 0, DsumCsq=0;
        for (int q=0; q<NumSizes; q++)
        {/*go through array and sum values for regression line*/
            /////SUM THE LOG OF THE COUNT*THE LOG OF THE SIZE\
            ////use the scale factor, calculated as the
            ////box size given divided by the largest size
            DsumSC
            = DsumSC + (log(1.0/(double)Sizes[q])*log((double)Counts[q]));
            ///SUM THE LOG OF THE SIZE
            DsumS = DsumS + log(1.0/(double)Sizes[q]);
            //SUM THE LOG OF THE COUNTS
            DsumC = DsumC + log((double)Counts[q]);
            //SUM THE LOG OF COUNTS TIMES LOG OF COUNTS
            DsumCsq
            =DsumCsq+(log((double)Counts[q])*log((double)Counts[q]));
            //SUM THE LOG OF SIZE TIMES LOG SIZE
            DsumSsq
            = DsumSsq + (log(1.0/(double)Sizes[q])*log(1.0/(double)Sizes[q]));
        }
        //CALCULATE THE SLOPE OF THE REGRESSION LINE
        double b=
        ((n*DsumSC)-(DsumS*DsumC))
        /((n*DsumSsq)-(DsumS*DsumS));
        double adf=b;//-1*b;
        //CALCULATE CORRELATION COEFFICIENT OF REGRESSION LINE
        double arSq= (n*(DsumSC)-(DsumS*DsumC))
        /Math.sqrt(((n*DsumSsq)-(DsumS*DsumS))
        *((n*DsumCsq)-((DsumC*DsumC))));
        
        arSq=arSq*arSq;
        
        //CALCULATE Y INTERCEPT OF REGRESSION LINE
        double ayint
        = (DsumC - (b*DsumS))/n;
        /*y intercept for linear regression by least squares*/
        //System.out.println(ayint);
        double F=Math.exp(ayint);//Prefactor A = e to the yint
        //for a power law: y=Ax^Df
        double StandardError=Math.sqrt((DsumCsq-ayint*DsumC-b*DsumSC)/(n-2));
        double [] DFinfo = new double[5];
        DFinfo[0]=adf;
        DFinfo[1]=arSq;
        DFinfo[2]=ayint;
        DFinfo[3]=1.0/F;//this is the inverse of the prefactor
        DFinfo[4]=StandardError;
        return DFinfo;
        
        
    }
    
    /** Converts to log base 10.
     * @param number double to convert
     * @return double log base 10 of number*/
    public double log(double number)
    {
        return Math.log(number)/Math.log(10.0);
    }
    
    /** Converts to log base 10.
     * @param number int to convert
     * @return double log base 10 of number*/
    public double log(int number)
    {
        return Math.log((double)number)/Math.log(10.0);
    }
    
    /** Converts to log base 10.
     * @param number float to convert
     * @return double log base 10 of number*/
    public double log(float number)
    {
        return Math.log((double)number)/Math.log(10.0);
    }
    
    /** Returns the negative slope of the regression line (box count
     * fractal dimension), the y intercept (for lacunarity), and the
     * correlation coefficient from the passed arrays without
     * converting to logs. Operates on the inverse of the sizes array.
     * @param Counts float array (e.g., number of boxes that
     * contained pixels when an image was scanned using many
     * series of box sizes, where one box size is applied over the
     * whole image without overlap and the number
     * that contained pixels is recorded for each scan)
     * @param Sizes float array (e.g., the corresponding
     * array of box sizes
     * @param NumSizes integer for the number of elements to
     * use in both arrays
     * @return double array with 5 entries: negative
     * slope of the regression line, r squared correlation
     * coefficient, y intercept, inverse of prefactor, and standard
     * error */
    public double[] LinearRegression(float[] Counts, int[] Sizes, int NumSizes)
    {  /*takes an array for box sizes and an array for Counts and
     *the number of box sizes to check and calculates stats using those arrays*/
        double n=(double)NumSizes;
        double DsumSC=0, DsumS=0, DsumC=0, DsumSsq = 0, DsumCsq=0;
        for (int q=0; q<NumSizes; q++)
        {/*go through array and sum values for regression line*/
            /////SUM THE COUNT*THE SIZE
            DsumSC
            = DsumSC + ((1.0/(double)Sizes[q])*(double)Counts[q]);
            ///SUM THE SIZE
            DsumS = DsumS + (1.0/(double)Sizes[q]);
            //SUM THE COUNTS
            DsumC = DsumC + ((double)Counts[q]);
            //SUM THE COUNTS TIMES COUNTS
            DsumCsq
            =DsumCsq+(((double)Counts[q])*((double)Counts[q]));
            //SUM THE SIZE TIMES SIZE
            DsumSsq
            = DsumSsq + ((1.0/(double)Sizes[q])*(1.0/(double)Sizes[q]));
        }
        //CALCULATE THE SLOPE OF THE REGRESSION LINE
        double b=
        ((n*DsumSC)-(DsumS*DsumC))
        /((n*DsumSsq)-(DsumS*DsumS));
        double adf=b;//-1*b;
        //CALCULATE CORRELATION COEFFICIENT OF REGRESSION LINE
        double arSq= (n*(DsumSC)-(DsumS*DsumC))
        /Math.sqrt(((n*DsumSsq)-(DsumS*DsumS))
        *((n*DsumCsq)-((DsumC*DsumC))));
        
        arSq=arSq*arSq;
        
        //CALCULATE Y INTERCEPT OF REGRESSION LINE
        double ayint
        = (DsumC - (b*DsumS))/n;
        /*y intercept for linear regression by least squares*/
        //System.out.println(ayint);
        double F=Math.exp(ayint);//Prefactor A = e to the yint
        //for a power law: y=Ax^Df
        double StandardError=Math.sqrt((DsumCsq-ayint*DsumC-b*DsumSC)/(n-2));
        
        double [] DFinfo = new double[5];
        DFinfo[0]=adf;
        DFinfo[1]=arSq;
        DFinfo[2]=ayint;
        DFinfo[3]=1.0/F;//this is the inverse of the prefactor
        DFinfo[4]=StandardError;
        return DFinfo;
        
        
    }
    
    public double[] PlainLinearRegression(float[] Y, int[] X, int Num)
    {  /*takes an array for box sizes and an array for Counts and
     *the number of box sizes to check and calculates stats using those arrays*/
        double n=(double)Num;
        double DsumSC=0, DsumS=0, DsumC=0, DsumSsq = 0, DsumCsq=0;
        for (int q=0; q<Num; q++)
        {/*go through array and sum values for regression line*/
            /////SUM THE COUNT*THE SIZE
            DsumSC
            = DsumSC + (X[q]*Y[q]);
            ///SUM THE SIZE
            DsumS = DsumS + (X[q]);
            //SUM THE COUNTS
            DsumC = DsumC + (Y[q]);
            //SUM THE COUNTS TIMES COUNTS
            DsumCsq
            =DsumCsq+(Y[q]*Y[q]);
            //SUM THE SIZE TIMES SIZE
            DsumSsq
            = DsumSsq + (X[q]*X[q]);
        }
        //CALCULATE THE SLOPE OF THE REGRESSION LINE
        double b=
        ((n*DsumSC)-(DsumS*DsumC))
        /((n*DsumSsq)-(DsumS*DsumS));
        double adf=b;//-1*b;
        //CALCULATE CORRELATION COEFFICIENT OF REGRESSION LINE
        double arSq= (n*(DsumSC)-(DsumS*DsumC))
        /Math.sqrt(((n*DsumSsq)-(DsumS*DsumS))
        *((n*DsumCsq)-((DsumC*DsumC))));
        
        arSq=arSq*arSq;
        
        //CALCULATE Y INTERCEPT OF REGRESSION LINE
        double ayint
        = (DsumC - (b*DsumS))/n;
        /*y intercept for linear regression by least squares*/
        //System.out.println(ayint);
        double F=Math.exp(ayint);//Prefactor A = e to the yint
        //for a power law: y=Ax^Df
        double StandardError=Math.sqrt((DsumCsq-ayint*DsumC-b*DsumSC)/(n-2));
        
        double [] DFinfo = new double[5];
        DFinfo[0]=adf;
        DFinfo[1]=arSq;
        DFinfo[2]=ayint;
        DFinfo[3]=1.0/F;//this is the inverse of the prefactor
        DFinfo[4]=StandardError;
        return DFinfo;
        
        
    }
    
    
    /**Returns a {@link #FindDfStats FindDfStats instance} from
     *integers arrays.   Calls FindDfStats for floats
     *after converting passed Counts array to floats.
     * @param Counts integer array
     * @param Sizes integer array
     * @param NumSizes integer for number of values to use
     * @return double array of values based on passed arrays
     */
    public double[] FindDfStats(int[] Counts, int[] Sizes, int NumSizes)
    {/*converter for int array instead of float array*/
        float[]c=new float[NumSizes];
        for (int w = 0; w<NumSizes; w++)  c[w] = (float)Counts[w];
        return FindDfStats(c,Sizes,NumSizes);
    }
    
}

